"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createCSRFToken = createCSRFToken;

var _web = require("./web");

async function createCSRFToken({
  options,
  cookieValue,
  isPost,
  bodyValue
}) {
  if (cookieValue) {
    const [csrfToken, csrfTokenHash] = cookieValue.split("|");
    const expectedCsrfTokenHash = await (0, _web.createHash)(`${csrfToken}${options.secret}`);

    if (csrfTokenHash === expectedCsrfTokenHash) {
      const csrfTokenVerified = isPost && csrfToken === bodyValue;
      return {
        csrfTokenVerified,
        csrfToken
      };
    }
  }

  const csrfToken = (0, _web.randomString)(32);
  const csrfTokenHash = await (0, _web.createHash)(`${csrfToken}${options.secret}`);
  const cookie = `${csrfToken}|${csrfTokenHash}`;
  return {
    cookie,
    csrfToken
  };
}