"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = getAuthorizationUrl;
exports.signCookie = signCookie;

var o = _interopRequireWildcard(require("@panva/oauth4webapi"));

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

async function getAuthorizationUrl({
  options,
  query
}) {
  var _provider$version, _provider$authorizati, _provider$checks, _provider$checks2, _provider$checks3;

  const {
    logger,
    provider
  } = options;

  if ((_provider$version = provider.version) !== null && _provider$version !== void 0 && _provider$version.startsWith("1.")) {
    const error = new Error("OAuth v1.x is not supported in next-auth/web");
    logger.error("GET_AUTHORIZATION_URL", error);
    throw error;
  }

  let url = (_provider$authorizati = provider.authorization) === null || _provider$authorizati === void 0 ? void 0 : _provider$authorizati.url;
  let as;

  if (!url) {
    const issuer = new URL(provider.issuer);
    const discoveryResponse = await o.discoveryRequest(issuer);
    const as = await o.processDiscoveryResponse(issuer, discoveryResponse);

    if (!as.authorization_endpoint) {
      throw new TypeError("Authorization server did not provide an authorization endpoint.");
    }

    url = new URL(as.authorization_endpoint);
  }

  const authParams = url.searchParams;
  const params = Object.assign({
    response_type: "code",
    client_id: provider.clientId,
    redirect_uri: provider.callbackUrl
  }, Object.fromEntries(authParams.entries()), query);

  for (const k in params) authParams.set(k, params[k]);

  const cookies = [];

  if ((_provider$checks = provider.checks) !== null && _provider$checks !== void 0 && _provider$checks.includes("state")) {
    const state = await createState(options);
    authParams.set("state", state.value);
    cookies.push(state);
  }

  if ((_provider$checks2 = provider.checks) !== null && _provider$checks2 !== void 0 && _provider$checks2.includes("pkce")) {
    var _as$code_challenge_me;

    if (as && !((_as$code_challenge_me = as.code_challenge_methods_supported) !== null && _as$code_challenge_me !== void 0 && _as$code_challenge_me.includes("S256"))) {
      provider.checks = ["nonce"];
    } else {
      const {
        code_challenge,
        pkce
      } = await createPKCE(options);
      authParams.set("code_challenge", code_challenge);
      authParams.set("code_challenge_method", "S256");
      cookies.push(pkce);
    }
  }

  if ((_provider$checks3 = provider.checks) !== null && _provider$checks3 !== void 0 && _provider$checks3.includes("nonce")) {
    const nonce = await createNonce(options);
    authParams.set("nonce", nonce.value);
    cookies.push(nonce);
  }

  logger.debug("GET_AUTHORIZATION_URL", {
    url,
    cookies,
    provider
  });
  return {
    redirect: url,
    cookies
  };
}

async function signCookie(type, value, maxAge, options) {
  const {
    cookies,
    jwt,
    logger
  } = options;
  logger.debug(`CREATE_${type.toUpperCase()}`, {
    value,
    maxAge
  });
  const expires = new Date();
  expires.setTime(expires.getTime() + maxAge * 1000);
  return {
    name: cookies[type].name,
    value: await jwt.encode({ ...jwt,
      maxAge,
      token: {
        value
      }
    }),
    options: { ...cookies[type].options,
      expires
    }
  };
}

const STATE_MAX_AGE = 60 * 15;

async function createState(options) {
  const raw = o.generateRandomState();
  const maxAge = STATE_MAX_AGE;
  const state = await signCookie("state", raw, maxAge, options);
  return state;
}

const PKCE_MAX_AGE = 60 * 15;

async function createPKCE(options) {
  const code_verifier = o.generateRandomCodeVerifier();
  const code_challenge = await o.calculatePKCECodeChallenge(code_verifier);
  const maxAge = PKCE_MAX_AGE;
  const pkce = await signCookie("pkceCodeVerifier", code_verifier, maxAge, options);
  return {
    code_challenge,
    pkce
  };
}

const NONCE_MAX_AGE = 60 * 15;

async function createNonce(options) {
  const raw = o.generateRandomNonce();
  const maxAge = NONCE_MAX_AGE;
  return await signCookie("nonce", raw, maxAge, options);
}