"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = parseProviders;

var _merge = require("../../utils/merge");

function parseProviders(params) {
  const {
    url,
    providerId
  } = params;
  const providers = params.providers.map(provider => {
    var _ref, _userOptions;

    let {
      options: userOptions,
      ...rest
    } = provider;

    if (provider.type === "oauth") {
      rest = normalizeOAuth(rest);
      userOptions = normalizeOAuth(userOptions);
    }

    const id = (_ref = (_userOptions = userOptions) === null || _userOptions === void 0 ? void 0 : _userOptions.id) !== null && _ref !== void 0 ? _ref : rest.id;
    return (0, _merge.merge)(rest, { ...userOptions,
      signinUrl: `${url}/signin/${id}`,
      callbackUrl: `${url}/callback/${id}`
    });
  });
  return {
    providers,
    provider: providers.find(({
      id
    }) => id === providerId)
  };
}

function normalizeOAuth(c) {
  var _c$version;

  if (!c) return {};
  const authorization = normalizeEndpoint(c.authorization);

  if (!((_c$version = c.version) !== null && _c$version !== void 0 && _c$version.startsWith("1."))) {
    var _c$idToken, _c$wellKnown$includes, _c$wellKnown, _authorization$url$se, _c$checks;

    (_c$idToken = c.idToken) !== null && _c$idToken !== void 0 ? _c$idToken : c.idToken = (_c$wellKnown$includes = (_c$wellKnown = c.wellKnown) === null || _c$wellKnown === void 0 ? void 0 : _c$wellKnown.includes("openid-configuration")) !== null && _c$wellKnown$includes !== void 0 ? _c$wellKnown$includes : authorization === null || authorization === void 0 ? void 0 : (_authorization$url$se = authorization.url.searchParams.get("scope")) === null || _authorization$url$se === void 0 ? void 0 : _authorization$url$se.includes("openid");
    (_c$checks = c.checks) !== null && _c$checks !== void 0 ? _c$checks : c.checks = ["state"];
    if (!Array.isArray(c.checks)) c.checks = [c.checks];
  }

  return { ...c,
    ...(authorization ? {
      authorization
    } : undefined),
    ...(c.token ? {
      token: normalizeEndpoint(c.token)
    } : undefined),
    ...(c.userinfo ? {
      userinfo: normalizeEndpoint(c.userinfo)
    } : undefined)
  };
}

function normalizeEndpoint(e) {
  if (!e) return;

  if (typeof e === "string") {
    return {
      url: new URL(e)
    };
  }

  const url = new URL(e.url);

  for (const k in e.params) url.searchParams.set(k, e.params[k]);

  return {
    url
  };
}